@@ -1,12 +1,15 @@
 import base64
 import datetime
+import hashlib
 import json
+import re
 from urllib.parse import parse_qs, urlencode, urlparse
 
 from django.contrib.auth import get_user_model
 from django.test import RequestFactory, TestCase
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.crypto import get_random_string
 from oauthlib.oauth2.rfc6749 import errors as oauthlib_errors
 
 from oauth2_provider.models import (
@@ -25,6 +28,9 @@ Grant = get_grant_model()
 RefreshToken = get_refresh_token_model()
 UserModel = get_user_model()
 
+URI_OOB = "urn:ietf:wg:oauth:2.0:oob"
+URI_OOB_AUTO = "urn:ietf:wg:oauth:2.0:oob:auto"
+
 
 # mocking a protected resource view
 class ResourceView(ProtectedResourceView):
@@ -40,16 +46,16 @@ class BaseTest(TestCase):
 
         oauth2_settings.ALLOWED_REDIRECT_URI_SCHEMES = ["http", "custom-scheme"]
 
-        self.application = Application(
+        self.application = Application.objects.create(
             name="Test Application",
             redirect_uris=(
                 "http://localhost http://example.com http://example.org custom-scheme://example.com"
+                " " + URI_OOB + " " + URI_OOB_AUTO
             ),
             user=self.dev_user,
             client_type=Application.CLIENT_CONFIDENTIAL,
             authorization_grant_type=Application.GRANT_AUTHORIZATION_CODE,
         )
-        self.application.save()
 
         oauth2_settings._SCOPES = ["read", "write"]
         oauth2_settings._DEFAULT_SCOPES = ["read", "write"]
@@ -537,6 +543,40 @@ class TestAuthorizationCodeTokenView(BaseTest):
         query_dict = parse_qs(urlparse(response["Location"]).query)
         return query_dict["code"].pop()
 
+    def generate_pkce_codes(self, algorithm, length=43):
+        """
+        Helper method to generate pkce codes
+        """
+        code_verifier = get_random_string(length)
+        if algorithm == "S256":
+            code_challenge = base64.urlsafe_b64encode(
+                hashlib.sha256(code_verifier.encode()).digest()
+            ).decode().rstrip("=")
+        else:
+            code_challenge = code_verifier
+        return code_verifier, code_challenge
+
+    def get_pkce_auth(self, code_challenge, code_challenge_method):
+        """
+        Helper method to retrieve a valid authorization code using pkce
+        """
+        oauth2_settings.PKCE_REQUIRED = True
+        authcode_data = {
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge": code_challenge,
+            "code_challenge_method": code_challenge_method,
+        }
+
+        response = self.client.post(reverse("oauth2_provider:authorize"), data=authcode_data)
+        query_dict = parse_qs(urlparse(response["Location"]).query)
+        oauth2_settings.PKCE_REQUIRED = False
+        return query_dict["code"].pop()
+
     def test_basic_auth(self):
         """
         Request an access token using basic authentication for client authentication
@@ -599,7 +639,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
 
         # check refresh token cannot be used twice
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
         content = json.loads(response.content.decode("utf-8"))
         self.assertTrue("invalid_grant" in content.values())
 
@@ -636,19 +676,24 @@ class TestAuthorizationCodeTokenView(BaseTest):
             "refresh_token": content["refresh_token"],
             "scope": content["scope"],
         }
+
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
         self.assertEqual(response.status_code, 200)
 
         content = json.loads(response.content.decode("utf-8"))
         self.assertTrue("access_token" in content)
         first_access_token = content["access_token"]
+        first_refresh_token = content["refresh_token"]
 
-        # check refresh token returns same data if used twice, see #497
+        # check access token returns same data if used twice, see #497
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
         self.assertEqual(response.status_code, 200)
         content = json.loads(response.content.decode("utf-8"))
         self.assertTrue("access_token" in content)
         self.assertEqual(content["access_token"], first_access_token)
+        # refresh token should be the same as well
+        self.assertTrue("refresh_token" in content)
+        self.assertEqual(content["refresh_token"], first_refresh_token)
         oauth2_settings.REFRESH_TOKEN_GRACE_PERIOD_SECONDS = 0
 
     def test_refresh_invalidates_old_tokens(self):
@@ -735,7 +780,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
             "scope": "read write nuke",
         }
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
 
     def test_refresh_fail_repeating_requests(self):
         """
@@ -763,7 +808,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
         self.assertEqual(response.status_code, 200)
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
 
     def test_refresh_repeating_requests(self):
         """
@@ -802,7 +847,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         rt.save()
 
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
         oauth2_settings.REFRESH_TOKEN_GRACE_PERIOD_SECONDS = 0
 
     def test_refresh_repeating_requests_non_rotating_tokens(self):
@@ -851,7 +896,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         auth_headers = get_basic_auth_header(self.application.client_id, self.application.client_secret)
 
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
 
     def test_basic_auth_bad_granttype(self):
         """
@@ -887,7 +932,7 @@ class TestAuthorizationCodeTokenView(BaseTest):
         auth_headers = get_basic_auth_header(self.application.client_id, self.application.client_secret)
 
         response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data, **auth_headers)
-        self.assertEqual(response.status_code, 401)
+        self.assertEqual(response.status_code, 400)
 
     def test_basic_auth_bad_secret(self):
         """
@@ -976,6 +1021,307 @@ class TestAuthorizationCodeTokenView(BaseTest):
         self.assertEqual(content["scope"], "read write")
         self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
 
+    def test_public_pkce_S256_authorize_get(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled. Tests if the authorize get is successfull
+        for the S256 algorithm
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        query_string = urlencode({
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge": code_challenge,
+            "code_challenge_method": "S256"
+        })
+        url = "{url}?{qs}".format(url=reverse("oauth2_provider:authorize"), qs=query_string)
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_plain_authorize_get(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled. Tests if the authorize get is successfull
+        for the plain algorithm
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("plain")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        query_string = urlencode({
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge": code_challenge,
+            "code_challenge_method": "plain"
+        })
+        url = "{url}?{qs}".format(url=reverse("oauth2_provider:authorize"), qs=query_string)
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_S256(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the S256 algorithm
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        authorization_code = self.get_pkce_auth(code_challenge, "S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id,
+            "code_verifier": code_verifier
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 200)
+
+        content = json.loads(response.content.decode("utf-8"))
+        self.assertEqual(content["token_type"], "Bearer")
+        self.assertEqual(content["scope"], "read write")
+        self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_plain(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the plain algorithm
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("plain")
+        authorization_code = self.get_pkce_auth(code_challenge, "plain")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id,
+            "code_verifier": code_verifier
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 200)
+
+        content = json.loads(response.content.decode("utf-8"))
+        self.assertEqual(content["token_type"], "Bearer")
+        self.assertEqual(content["scope"], "read write")
+        self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_invalid_algorithm(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with an invalid algorithm
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("invalid")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        query_string = urlencode({
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge": code_challenge,
+            "code_challenge_method": "invalid",
+        })
+        url = "{url}?{qs}".format(url=reverse("oauth2_provider:authorize"), qs=query_string)
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertIn("error=invalid_request", response["Location"])
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_missing_code_challenge(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled but with the code_challenge missing
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.skip_authorization = True
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        query_string = urlencode({
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge_method": "S256"
+        })
+        url = "{url}?{qs}".format(url=reverse("oauth2_provider:authorize"), qs=query_string)
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assertIn("error=invalid_request", response["Location"])
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_missing_code_challenge_method(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled but with the code_challenge_method missing
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        query_string = urlencode({
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": "http://example.org",
+            "response_type": "code",
+            "allow": True,
+            "code_challenge": code_challenge
+        })
+        url = "{url}?{qs}".format(url=reverse("oauth2_provider:authorize"), qs=query_string)
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_S256_invalid_code_verifier(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the S256 algorithm and an invalid code_verifier
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        authorization_code = self.get_pkce_auth(code_challenge, "S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id,
+            "code_verifier": "invalid"
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 400)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_plain_invalid_code_verifier(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the plain algorithm and an invalid code_verifier
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("plain")
+        authorization_code = self.get_pkce_auth(code_challenge, "plain")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id,
+            "code_verifier": "invalid"
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 400)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_S256_missing_code_verifier(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the S256 algorithm and the code_verifier missing
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("S256")
+        authorization_code = self.get_pkce_auth(code_challenge, "S256")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 400)
+        oauth2_settings.PKCE_REQUIRED = False
+
+    def test_public_pkce_plain_missing_code_verifier(self):
+        """
+        Request an access token using client_type: public
+        and PKCE enabled with the plain algorithm and the code_verifier missing
+        """
+        self.client.login(username="test_user", password="123456")
+
+        self.application.client_type = Application.CLIENT_PUBLIC
+        self.application.save()
+        code_verifier, code_challenge = self.generate_pkce_codes("plain")
+        authorization_code = self.get_pkce_auth(code_challenge, "plain")
+        oauth2_settings.PKCE_REQUIRED = True
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": "http://example.org",
+            "client_id": self.application.client_id
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 400)
+        oauth2_settings.PKCE_REQUIRED = False
+
     def test_malicious_redirect_uri(self):
         """
         Request an access token using client_type: public and ensure redirect_uri is
@@ -1105,6 +1451,92 @@ class TestAuthorizationCodeTokenView(BaseTest):
         self.assertEqual(content["scope"], "read write")
         self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
 
+    def test_oob_as_html(self):
+        """
+        Test out-of-band authentication.
+        """
+        self.client.login(username="test_user", password="123456")
+
+        authcode_data = {
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": URI_OOB,
+            "response_type": "code",
+            "allow": True,
+        }
+
+        response = self.client.post(reverse("oauth2_provider:authorize"), data=authcode_data)
+        self.assertEqual(response.status_code, 200)
+        self.assertRegex(response["Content-Type"], r"^text/html")
+
+        content = response.content.decode("utf-8")
+
+        # "A lot of applications, for legacy reasons, use this and regex
+        # to extract the token, risking summoning zalgo in the process."
+        #    -- https://github.com/jazzband/django-oauth-toolkit/issues/235
+
+        matches = re.search(r".*<code>([^<>]*)</code>", content)
+        self.assertIsNotNone(matches, msg="OOB response contains code inside <code> tag")
+        self.assertEqual(len(matches.groups()), 1, msg="OOB response contains multiple <code> tags")
+        authorization_code = matches.groups()[0]
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": URI_OOB,
+            "client_id": self.application.client_id,
+            "client_secret": self.application.client_secret,
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 200)
+
+        content = json.loads(response.content.decode("utf-8"))
+        self.assertEqual(content["token_type"], "Bearer")
+        self.assertEqual(content["scope"], "read write")
+        self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
+
+    def test_oob_as_json(self):
+        """
+        Test out-of-band authentication, with a JSON response.
+        """
+        self.client.login(username="test_user", password="123456")
+
+        authcode_data = {
+            "client_id": self.application.client_id,
+            "state": "random_state_string",
+            "scope": "read write",
+            "redirect_uri": URI_OOB_AUTO,
+            "response_type": "code",
+            "allow": True,
+        }
+
+        response = self.client.post(reverse("oauth2_provider:authorize"), data=authcode_data)
+        self.assertEqual(response.status_code, 200)
+        self.assertRegex(response["Content-Type"], "^application/json")
+
+        parsed_response = json.loads(response.content.decode("utf-8"))
+
+        self.assertIn("access_token", parsed_response)
+        authorization_code = parsed_response["access_token"]
+
+        token_request_data = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "redirect_uri": URI_OOB_AUTO,
+            "client_id": self.application.client_id,
+            "client_secret": self.application.client_secret,
+        }
+
+        response = self.client.post(reverse("oauth2_provider:token"), data=token_request_data)
+        self.assertEqual(response.status_code, 200)
+
+        content = json.loads(response.content.decode("utf-8"))
+        self.assertEqual(content["token_type"], "Bearer")
+        self.assertEqual(content["scope"], "read write")
+        self.assertEqual(content["expires_in"], oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
+
 
 class TestAuthorizationCodeProtectedResource(BaseTest):
     def test_resource_access_allowed(self):
