@@ -81,14 +81,13 @@ class TestTokenIntrospectionAuth(TestCase):
             "resource_server", "test@example.com", "123456"
         )
 
-        self.application = Application(
+        self.application = Application.objects.create(
             name="Test Application",
             redirect_uris="http://localhost http://example.com http://example.org",
             user=self.resource_server_user,
             client_type=Application.CLIENT_CONFIDENTIAL,
             authorization_grant_type=Application.GRANT_AUTHORIZATION_CODE,
         )
-        self.application.save()
 
         self.resource_server_token = AccessToken.objects.create(
             user=self.resource_server_user, token="12345678900",
